import std/asyncdispatch
import std/strformat
import std/strutils
import std/times
import nim_gpiod
import ../lib/asyncsync
import ../lib/config_parser

type
  DIEvent* {.pure.} = enum
    Init = "Init"
    Changed = "Changed"
  MsgDI* = object
    timestamp*: int64
    name*: string
    state*: int
    event*: DIEvent
  TaskDIObj = object
    di: DIParam
    aq: AsyncQueue[MsgDI]
    gpio: AsyncGpio
  TaskDI* = ref TaskDIObj

# ------------------------------------------------------------------------------
#
# ------------------------------------------------------------------------------
proc newTaskDI*(param: DIParam, aq: AsyncQueue[MsgDI]): TaskDI =
  result = new TaskDI
  result.di = param
  result.aq = aq
  result.gpio = newGpio(param.signal, "DI_MQTT")

# ------------------------------------------------------------------------------
#
# ------------------------------------------------------------------------------
proc get_timestamp(): int64 =
  result = now().toTime().toUnix()

# ------------------------------------------------------------------------------
#
# ------------------------------------------------------------------------------
proc `$`*(msg: MsgDI): string =
  var buf = newSeqOfCap[string](4)
  buf.add(&"--- GPIO: {msg.name}")
  buf.add(&"  * event: {msg.event}")
  buf.add(&"  * state: {msg.state}")
  buf.add(&"  * timestamp: {msg.timestamp}")
  result = buf.join("\n")

# ------------------------------------------------------------------------------
#
# ------------------------------------------------------------------------------
proc run*(self: TaskDI) {.async.} =
  let val_init = self.gpio.get_value()
  let msg = MsgDI(timestamp: get_timestamp(), name: self.di.name,
      state: val_init, event: DIEvent.Init)
  await self.aq.put(msg)

  var current = val_init
  while true:
    let event = await self.gpio.wait_event(Edge.Both, 10)
    let msg = MsgDI(timestamp: get_timestamp(), name: self.di.name,
        state: event.value, event: DIEvent.Changed)
    await self.aq.put(msg)
    current = event.value


when isMainModule:
  proc main() {.async.} =
    let aq = newAsyncQueue[MsgDI](32)
    var tasks = newSeqOfCap[TaskDI](12)
    for i in 0..<12:
      let
        name = &"DI_{i}"
        signal = &"EXT_DI{i}"
      let param = DIParam(name: name, signal: signal)
      let task = newTaskDI(param, aq)
      asyncCheck task.run()
      tasks.add(task)

    while true:
      let msg = await aq.get()
      echo msg

  waitFor main()
